@@ -2,6 +2,7 @@
 
 from AccessControl import ClassSecurityInfo
 from DateTime import DateTime
+from Products.Archetypes.interfaces import IBaseObject
 from Products.Archetypes.Registry import registerWidget
 from Products.Archetypes.Widget import TypesWidget
 from Products.CMFCore.utils import getToolByName
@@ -10,13 +11,17 @@ from plone.app.widgets.base import InputWidget
 from plone.app.widgets.base import SelectWidget
 from plone.app.widgets.base import TextareaWidget
 from plone.app.widgets.base import dict_merge
+from plone.app.widgets.interfaces import IFieldPermissionChecker
 from plone.app.widgets.utils import NotImplemented
 from plone.app.widgets.utils import get_date_options
 from plone.app.widgets.utils import get_datetime_options
 from plone.app.widgets.utils import get_ajaxselect_options
 from plone.app.widgets.utils import get_relateditems_options
 from plone.app.widgets.utils import get_querystring_options
+from plone.app.widgets.utils import get_tinymce_options
 from plone.uuid.interfaces import IUUID
+from zope.interface import implements
+from zope.component import adapts
 
 import json
 
@@ -125,6 +130,12 @@ class DateWidget(BaseWidget):
             get_date_options(request),
             args['pattern_options'])
 
+        if 'date' in args['pattern_options'] and \
+           'firstDay' in args['pattern_options']['date'] and \
+           callable(args['pattern_options']['date']['firstDay']):
+            args['pattern_options']['date']['firstDay'] = \
+                args['pattern_options']['date']['firstDay']()
+
         return args
 
     security = ClassSecurityInfo()
@@ -133,12 +144,14 @@ class DateWidget(BaseWidget):
     def process_form(self, instance, field, form, empty_marker=None):
         """Basic impl for form processing in a widget"""
 
-        value = form.get(field.getName(), None)
-        if not value:
+        value = form.get(field.getName(), empty_marker)
+        if value is empty_marker:
             return empty_marker
 
+        value = value.split('-')
+
         try:
-            value = DateTime(datetime(*map(int, value.split('-'))))
+            value = DateTime(datetime(*map(int, value)))
         except:
             return empty_marker
 
@@ -219,20 +232,23 @@ class DatetimeWidget(DateWidget):
     def process_form(self, instance, field, form, empty_marker=None):
         """Basic impl for form processing in a widget"""
 
-        value = form.get(field.getName(), None)
-        if not value:
-            return empty_marker, {}
+        value = form.get(field.getName(), empty_marker)
+        if value is empty_marker:
+            return empty_marker
 
         tmp = value.split(' ')
         if not tmp[0]:
             return empty_marker
         value = tmp[0].split('-')
-        value += tmp[1].split(':')
+        if len(tmp) == 2 and ':' in tmp[1]:
+            value += tmp[1].split(':')
+        else:
+            value += ['00', '00']
 
         try:
             value = DateTime(datetime(*map(int, value)))
         except:
-            return empty_marker, {}
+            return empty_marker
 
         return value, {}
 
@@ -256,6 +272,7 @@ class SelectWidget(BaseWidget):
         'pattern_options': {},
         'separator': ';',
         'multiple': False,
+        'orderable': False,
     })
 
     def _base_args(self, context, field, request):
@@ -288,8 +305,21 @@ class SelectWidget(BaseWidget):
         if self.separator:
             args['pattern_options']['separator'] = self.separator
 
+        if self.multiple and self.orderable:
+            args['pattern_options']['orderable'] = True
+
         return args
 
+    security = ClassSecurityInfo()
+    security.declarePublic('process_form')
+
+    def process_form(self, instance, field, form, empty_marker=None):
+        value = form.get(field.getName(), empty_marker)
+        if value is empty_marker:
+            return empty_marker
+        if self.multiple and isinstance(value, basestring):
+            value = value.strip().split(self.separator)
+        return value, {}
 
 registerWidget(
     SelectWidget,
@@ -311,6 +341,7 @@ class AjaxSelectWidget(BaseWidget):
         'separator': ';',
         'vocabulary': None,
         'vocabulary_view': '@@getVocabulary',
+        'orderable': False,
     })
 
     def _base_args(self, context, field, request):
@@ -328,9 +359,13 @@ class AjaxSelectWidget(BaseWidget):
         args.setdefault('pattern_options', {})
         args['pattern_options'] = dict_merge(
             get_ajaxselect_options(context, args['value'], self.separator,
-                                   self.vocabulary, self.vocabulary_view),
+                                   self.vocabulary, self.vocabulary_view,
+                                   field.getName()),
             args['pattern_options'])
 
+        if self.orderable:
+            args['pattern_options']['orderable'] = True
+
         return args
 
     security = ClassSecurityInfo()
@@ -351,6 +386,57 @@ registerWidget(
     used_for=('Products.Archetypes.Field.LinesField',))
 
 
+class KeywordsWidget(AjaxSelectWidget):
+    """Keywords widget for Archetypes."""
+
+    _base = InputWidget
+
+    _properties = AjaxSelectWidget._properties.copy()
+    _properties.update({
+        'vocabulary': 'plone.app.vocabularies.Keywords',
+    })
+
+    def _base_args(self, context, field, request):
+        args = super(KeywordsWidget, self)._base_args(context, field,
+                                                      request)
+
+        membership = getToolByName(context, 'portal_membership')
+        user = membership.getAuthenticatedMember()
+
+        site_properties = getToolByName(
+            context, 'portal_properties')['site_properties']
+        allowRolesToAddKeywords = site_properties.getProperty(
+            'allowRolesToAddKeywords', None)
+
+        allowNewItems = False
+        if allowRolesToAddKeywords and [
+            role for role in user.getRolesInContext(context)
+                if role in allowRolesToAddKeywords]:
+            allowNewItems = True
+
+        args.setdefault('pattern_options', {})
+        args['pattern_options']['allowNewItems'] = allowNewItems
+
+        return args
+
+    security = ClassSecurityInfo()
+    security.declarePublic('process_form')
+
+    def process_form(self, instance, field, form, empty_marker=None):
+        value = form.get(field.getName(), empty_marker)
+        if value is empty_marker:
+            return empty_marker
+        value = value.strip().split(self.separator)
+        return value, {}
+
+
+registerWidget(
+    KeywordsWidget,
+    title='Keywords widget',
+    description=('Keywords widget'),
+    used_for=('Products.Archetypes.Field.LinesField',))
+
+
 class RelatedItemsWidget(BaseWidget):
     """Related items widget for Archetypes."""
 
@@ -363,6 +449,7 @@ class RelatedItemsWidget(BaseWidget):
         'separator': ';',
         'vocabulary': 'plone.app.vocabularies.Catalog',
         'vocabulary_view': '@@getVocabulary',
+        'allow_sorting': True,
     })
 
     def _base_args(self, context, field, request):
@@ -384,9 +471,11 @@ class RelatedItemsWidget(BaseWidget):
         args['value'] = self.separator.join(value)
 
         args.setdefault('pattern_options', {})
+        args['pattern_options']['orderable'] = self.allow_sorting
         args['pattern_options'] = dict_merge(
             get_relateditems_options(context, args['value'], self.separator,
-                                     self.vocabulary, self.vocabulary_view),
+                                     self.vocabulary, self.vocabulary_view,
+                                     field.getName()),
             args['pattern_options'])
 
         return args
@@ -476,6 +565,12 @@ class TinyMCEWidget(BaseWidget):
         args['value'] = (request.get(field.getName(),
                                      field.getAccessor(context)())
                          ).decode(charset)
+
+        args.setdefault('pattern_options', {})
+        merged = dict_merge(get_tinymce_options(context, field, request),
+                            args['pattern_options'])
+        args['pattern_options'] = merged['pattern_options']
+
         return args
 
 
@@ -484,3 +579,23 @@ registerWidget(
     title='TinyMCE widget',
     description=('TinyMCE widget'),
     used_for='Products.Archetypes.Field.TextField')
+
+
+class ATFieldPermissionChecker(object):
+    implements(IFieldPermissionChecker)
+    adapts(IBaseObject)
+
+    def __init__(self, context):
+        self.context = context
+
+    def validate(self, field_name, vocabulary_name=None):
+        field = self.context.getField(field_name)
+        if field is not None:
+            # If a vocabulary name was specified and it doesn't match
+            # the value for the field or the widget, fail.
+            if vocabulary_name and (
+               vocabulary_name != getattr(field.widget, 'vocabulary', None) and
+               vocabulary_name != getattr(field, 'vocabulary_factory', None)):
+                return False
+            return field.checkPermission('w', self.context)
+        raise AttributeError('No such field: {}'.format(field_name))
