@@ -1,41 +1,77 @@
 # -*- coding: utf-8 -*-
 
+from AccessControl import getSecurityManager
 from Products.CMFCore.utils import getToolByName
 from datetime import date
 from datetime import datetime
+
+from plone.app.textfield.widget import IRichTextWidget \
+    as patextfield_IRichTextWidget
+from plone.app.textfield.widget import RichTextWidget \
+    as patextfield_RichTextWidget
+from plone.app.textfield.value import RichTextValue
 from plone.app.widgets.base import InputWidget
-from plone.app.widgets.base import SelectWidget
+from plone.app.widgets.base import SelectWidget as BaseSelectWidget
 from plone.app.widgets.base import TextareaWidget
 from plone.app.widgets.base import dict_merge
+from plone.app.widgets.interfaces import IFieldPermissionChecker
+from plone.app.widgets.interfaces import IWidgetsLayer
 from plone.app.widgets.utils import NotImplemented
 from plone.app.widgets.utils import get_ajaxselect_options
 from plone.app.widgets.utils import get_date_options
 from plone.app.widgets.utils import get_datetime_options
 from plone.app.widgets.utils import get_querystring_options
 from plone.app.widgets.utils import get_relateditems_options
+from plone.app.widgets.utils import get_tinymce_options
+from plone.autoform.interfaces import WIDGETS_KEY
+from plone.autoform.interfaces import WRITE_PERMISSIONS_KEY
+from plone.autoform.utils import resolveDottedName
+from plone.dexterity.interfaces import IDexterityContent
+from plone.dexterity.utils import iterSchemata, getAdditionalSchemata
+from plone.supermodel.utils import mergedTaggedValueDict
 from plone.uuid.interfaces import IUUID
 from z3c.form.browser.select import SelectWidget as z3cform_SelectWidget
+from z3c.form.browser.text import TextWidget as z3cform_TextWidget
+from z3c.form.browser.widget import HTMLInputWidget
 from z3c.form.converter import BaseDataConverter
+from z3c.form.converter import CollectionSequenceDataConverter
+from z3c.form.interfaces import IAddForm
 from z3c.form.interfaces import IFieldWidget
 from z3c.form.interfaces import IFormLayer
 from z3c.form.interfaces import ISelectWidget
-from z3c.form.interfaces import ITextAreaWidget
 from z3c.form.interfaces import ITextWidget
+from z3c.form.interfaces import NO_VALUE
+from z3c.form.util import getSpecification
 from z3c.form.widget import FieldWidget
 from z3c.form.widget import Widget
 from zope.component import adapter
 from zope.component import adapts
+from zope.component import queryMultiAdapter
+from zope.component import queryUtility
+from zope.component.hooks import getSite
+from zope.i18n import translate
 from zope.interface import implementer
+from zope.interface import implements
 from zope.interface import implementsOnly
+from zope.publisher.browser import TestRequest
+from zope.schema.interfaces import IChoice
 from zope.schema.interfaces import ICollection
 from zope.schema.interfaces import IDate
 from zope.schema.interfaces import IDatetime
-from zope.schema.interfaces import IField
 from zope.schema.interfaces import IList
+from zope.schema.interfaces import ISequence
+from zope.security.interfaces import IPermission
 
 import pytz
 import json
 
+try:
+    from plone.app.contenttypes.behaviors.collection import ICollection as IDXCollection  # noqa
+    from plone.app.contenttypes.behaviors.richtext import IRichText  # noqa
+    HAS_PAC = True
+except ImportError:
+    HAS_PAC = False
+
 
 class IDateField(IDate):
     """Marker interface for the DateField."""
@@ -69,7 +105,7 @@ class IRelatedItemsWidget(ITextWidget):
     """Marker interface for the RelatedItemsWidget."""
 
 
-class ITinyMCEWidget(ITextAreaWidget):
+class IRichTextWidget(patextfield_IRichTextWidget):
     """Marker interface for the TinyMCEWidget."""
 
 
@@ -140,7 +176,10 @@ class DatetimeWidgetConverter(BaseDataConverter):
         if not tmp[0]:
             return self.field.missing_value
         value = tmp[0].split('-')
-        value += tmp[1].split(':')
+        if len(tmp) == 2 and ':' in tmp[1]:
+            value += tmp[1].split(':')
+        else:
+            value += ['00', '00']
 
         # Eventually set timezone
         old_val = getattr(self.widget.context, self.field.getName(), None)
@@ -165,6 +204,33 @@ class DatetimeWidgetConverter(BaseDataConverter):
         return datetime(*map(int, value))
 
 
+class SelectWidgetConverter(CollectionSequenceDataConverter):
+    """Data converter for Select widget."""
+
+    adapts(ICollection, ISelectWidget)
+
+    def toFieldValue(self, value):
+        """Converts from widget value to field.
+
+        :param value: Value inserted by Select2 widget or default html
+                      select/multi-select
+        :type value: string | list
+
+        :returns: List of items
+        :rtype: list | tuple | set
+        """
+        separator = getattr(self.widget, 'separator', ';')
+        if isinstance(value, basestring):
+            value = value.strip()
+            if value:
+                value = value.split(separator)
+            else:
+                return self.field.missing_value
+        elif value == (u'',):
+            return self.field.missing_value
+        return super(SelectWidgetConverter, self).toFieldValue(value)
+
+
 class AjaxSelectWidgetConverter(BaseDataConverter):
     """Data converter for ICollection."""
 
@@ -245,7 +311,10 @@ class RelatedItemsDataConverter(BaseDataConverter):
         value = value.split(separator)
         value = [v.split('/')[0] for v in value]
 
-        catalog = getToolByName(self.widget.context, 'portal_catalog')
+        try:
+            catalog = getToolByName(self.widget.context, 'portal_catalog')
+        except AttributeError:
+            catalog = getToolByName(getSite(), 'portal_catalog')
 
         return collectionType(item.getObject()
                               for item in catalog(UID=value) if item)
@@ -265,8 +334,8 @@ class QueryStringDataConverter(BaseDataConverter):
         :returns: Query string converted to JSON.
         :rtype: string
         """
-        if value is self.field.missing_value:
-            return self.field.missing_value
+        if not value:
+            return '[]'
         return json.dumps(value)
 
     def toFieldValue(self, value):
@@ -278,9 +347,13 @@ class QueryStringDataConverter(BaseDataConverter):
         :returns: Query string.
         :rtype: list
         """
-        if value is self.field.missing_value:
+        try:
+            value = json.loads(value)
+        except ValueError:
+            value = None
+        if not value:
             return self.field.missing_value
-        return json.loads(value)
+        return value
 
 
 class BaseWidget(Widget):
@@ -307,7 +380,7 @@ class BaseWidget(Widget):
             raise NotImplemented("'pattern' option is not provided.")
         return {
             'pattern': self.pattern,
-            'pattern_options': self.pattern_options,
+            'pattern_options': self.pattern_options.copy(),
         }
 
     def render(self):
@@ -321,7 +394,7 @@ class BaseWidget(Widget):
         return self._base(**self._base_args()).render()
 
 
-class DateWidget(BaseWidget):
+class DateWidget(BaseWidget, HTMLInputWidget):
     """Date widget for z3c.form."""
 
     _base = InputWidget
@@ -371,7 +444,7 @@ class DateWidget(BaseWidget):
 
         field_value = self._converter(
             self.field, self).toFieldValue(self.value)
-        if field_value is self.fields.missing_value:
+        if field_value is self.field.missing_value:
             return u''
 
         formatter = self.request.locale.dates.getFormatter(
@@ -384,7 +457,7 @@ class DateWidget(BaseWidget):
         return field_value.ctime()
 
 
-class DatetimeWidget(DateWidget):
+class DatetimeWidget(DateWidget, HTMLInputWidget):
     """Datetime widget for z3c.form."""
 
     _converter = DatetimeWidgetConverter
@@ -424,13 +497,16 @@ class DatetimeWidget(DateWidget):
 class SelectWidget(BaseWidget, z3cform_SelectWidget):
     """Select widget for z3c.form."""
 
-    _base = SelectWidget
+    _base = BaseSelectWidget
 
     implementsOnly(ISelectWidget)
 
     pattern = 'select2'
     pattern_options = BaseWidget.pattern_options.copy()
+
+    separator = ';'
     multiple = False
+    orderable = False
 
     def _base_args(self):
         """Method which will calculate _base class arguments.
@@ -453,13 +529,41 @@ class SelectWidget(BaseWidget, z3cform_SelectWidget):
 
         items = []
         for item in self.items():
+            if not isinstance(item['content'], basestring):
+                item['content'] = translate(
+                    item['content'],
+                    context=self.request,
+                    default=item['value'])
             items.append((item['value'], item['content']))
         args['items'] = items
 
+        options = args.setdefault('pattern_options', {})
+        if self.multiple or ICollection.providedBy(self.field):
+            options['multiple'] = args['multiple'] = self.multiple = True
+
+        # ISequence represents an orderable collection
+        if ISequence.providedBy(self.field) or self.orderable:
+            options['orderable'] = True
+
+        if self.multiple:
+            options['separator'] = self.separator
+
         return args
 
+    def extract(self, default=NO_VALUE):
+        """Override extract to handle delimited response values.
+        Skip the vocabulary validation provided in the parent
+        method, since it's not ever done for single selects."""
+        if (self.name not in self.request and
+                self.name + '-empty-marker' in self.request):
+            return []
+        value = self.request.get(self.name, default)
+        if not isinstance(value, (tuple, list)):
+            value = (value,)
+        return value
 
-class AjaxSelectWidget(BaseWidget):
+
+class AjaxSelectWidget(BaseWidget, z3cform_TextWidget):
     """Ajax select widget for z3c.form."""
 
     _base = InputWidget
@@ -472,6 +576,12 @@ class AjaxSelectWidget(BaseWidget):
     separator = ';'
     vocabulary = None
     vocabulary_view = '@@getVocabulary'
+    orderable = False
+
+    def update(self, *args, **kwargs):
+        if not hasattr(self, 'vocabulary'):
+            self.vocabulary = getattr(self.field, 'vocabularyName', None)
+        z3cform_TextWidget.update(self, *args, **kwargs)
 
     def _base_args(self):
         """Method which will calculate _base class arguments.
@@ -488,23 +598,32 @@ class AjaxSelectWidget(BaseWidget):
 
         args = super(AjaxSelectWidget, self)._base_args()
 
-        vocabulary_factory = getattr(self.field, 'vocabulary_factory', None)
-        if not self.vocabulary:
-            self.vocabulary = vocabulary_factory
-
         args['name'] = self.name
         args['value'] = self.value
 
         args.setdefault('pattern_options', {})
+
+        field_name = self.field and self.field.__name__ or None
+
+        context = self.context
+        # We need special handling for AddForms
+        if IAddForm.providedBy(getattr(self, 'form')):
+            context = self.form
+
         args['pattern_options'] = dict_merge(
-            get_ajaxselect_options(self.context, args['value'], self.separator,
-                                   self.vocabulary, self.vocabulary_view),
+            get_ajaxselect_options(context, args['value'], self.separator,
+                                   self.vocabulary, self.vocabulary_view,
+                                   field_name),
             args['pattern_options'])
 
+        # ISequence represents an orderable collection
+        if ISequence.providedBy(self.field) or self.orderable:
+            args['pattern_options']['orderable'] = True
+
         return args
 
 
-class RelatedItemsWidget(BaseWidget):
+class RelatedItemsWidget(BaseWidget, z3cform_TextWidget):
     """RelatedItems widget for z3c.form."""
 
     _base = InputWidget
@@ -515,8 +634,19 @@ class RelatedItemsWidget(BaseWidget):
     pattern_options = BaseWidget.pattern_options.copy()
 
     separator = ';'
-    vocabulary = 'plone.app.vocabularies.Catalog'
+    vocabulary = None
     vocabulary_view = '@@getVocabulary'
+    orderable = False
+
+    def update(self, *args, **kwargs):
+        value_type = getattr(self.field, 'value_type', None)
+        if value_type:
+            self.vocabulary = getattr(value_type,
+                                      'vocabularyName',
+                                      'plone.app.vocabularies.Catalog')
+        if self.vocabulary is None:
+            self.vocabulary = 'plone.app.vocabularies.Catalog'
+        z3cform_TextWidget.update(self, *args, **kwargs)
 
     def _base_args(self):
         """Method which will calculate _base class arguments.
@@ -530,27 +660,25 @@ class RelatedItemsWidget(BaseWidget):
         :returns: Arguments which will be passed to _base
         :rtype: dict
         """
-
         args = super(RelatedItemsWidget, self)._base_args()
 
-        vocabulary_factory = getattr(self.field, 'vocabulary_factory', None)
-        if not self.vocabulary:
-            self.vocabulary = vocabulary_factory
-
         args['name'] = self.name
         args['value'] = self.value
 
         args.setdefault('pattern_options', {})
+        if IChoice.providedBy(self.field):
+            args['pattern_options']['maximumSelectionSize'] = 1
+        field_name = self.field and self.field.__name__ or None
         args['pattern_options'] = dict_merge(
             get_relateditems_options(self.context, args['value'],
                                      self.separator, self.vocabulary,
-                                     self.vocabulary_view),
+                                     self.vocabulary_view, field_name),
             args['pattern_options'])
 
         return args
 
 
-class QueryStringWidget(BaseWidget):
+class QueryStringWidget(BaseWidget, z3cform_TextWidget):
     """QueryString widget for z3c.form."""
 
     _base = InputWidget
@@ -586,43 +714,159 @@ class QueryStringWidget(BaseWidget):
         return args
 
 
-class TinyMCEWidget(BaseWidget):
+class RichTextWidget(BaseWidget, patextfield_RichTextWidget):
     """TinyMCE widget for z3c.form."""
 
     _base = TextareaWidget
 
-    implementsOnly(ITextAreaWidget)
+    implementsOnly(IRichTextWidget)
 
     pattern = 'tinymce'
     pattern_options = BaseWidget.pattern_options.copy()
 
     def _base_args(self):
-        """Method which will calculate _base class arguments.
-
-        Returns (as python dictionary):
-            - `pattern`: pattern name
-            - `pattern_options`: pattern options
-            - `name`: field name
-            - `value`: field value
-
-        :returns: Arguments which will be passed to _base
-        :rtype: dict
-        """
-        args = super(TinyMCEWidget, self)._base_args()
-
+        args = super(RichTextWidget, self)._base_args()
         args['name'] = self.name
-        args['value'] = self.value
+        properties = getToolByName(self.context, 'portal_properties')
+        charset = properties.site_properties.getProperty('default_charset',
+                                                         'utf-8')
+        value = self.value and self.value.raw_encoded or ''
+        args['value'] = (self.request.get(
+            self.field.getName(), value)).decode(charset)
+
+        args.setdefault('pattern_options', {})
+        merged = dict_merge(get_tinymce_options(self.context, self.field, self.request),  # noqa
+                            args['pattern_options'])
+        args['pattern_options'] = merged['pattern_options']
 
         return args
 
+    def render(self):
+        """Render widget.
+
+        :returns: Widget's HTML.
+        :rtype: string
+        """
+        if self.mode != 'display':
+            return super(RichTextWidget, self).render()
+
+        if not self.value:
+            return ''
+
+        if isinstance(self.value, RichTextValue):
+            return self.value.output
+
+        return super(RichTextWidget, self).render()
+
 
-@adapter(IField, IFormLayer)
 @implementer(IFieldWidget)
 def DateFieldWidget(field, request):
     return FieldWidget(field, DateWidget(request))
 
 
-@adapter(IField, IFormLayer)
 @implementer(IFieldWidget)
 def DatetimeFieldWidget(field, request):
     return FieldWidget(field, DatetimeWidget(request))
+
+
+@implementer(IFieldWidget)
+def RelatedItemsFieldWidget(field, request):
+    # TODO: when field is type IRelationChoice configure widget to only allow
+    # one item to be selected
+    return FieldWidget(field, RelatedItemsWidget(request))
+
+
+if HAS_PAC:
+    @adapter(getSpecification(IDXCollection['query']), IFormLayer)
+    @implementer(IFieldWidget)
+    def QueryStringFieldWidget(field, request):
+        return FieldWidget(field, QueryStringWidget(request))
+
+    @adapter(getSpecification(IRichText['text']), IFormLayer)
+    @implementer(IFieldWidget)
+    def RichTextFieldWidget(field, request):
+        return FieldWidget(field, RichTextWidget(request))
+
+
+class MockRequest(TestRequest):
+    implements(IWidgetsLayer)
+
+
+class DXFieldPermissionChecker(object):
+    """
+    """
+
+    implements(IFieldPermissionChecker)
+    adapts(IDexterityContent)
+
+    DEFAULT_PERMISSION = 'Modify portal content'
+
+    def __init__(self, context):
+        self.context = context
+        self._request = MockRequest()
+
+    def _get_schemata(self):
+        return iterSchemata(self.context)
+
+    def validate(self, field_name, vocabulary_name=None):
+        context = self.context
+        checker = getSecurityManager().checkPermission
+        schemata = self._get_schemata()
+        for schema in schemata:
+            if field_name in schema:
+                # If a vocabulary name was specified and it does not
+                # match the vocabulary name for the field or widget,
+                # fail.
+                field = schema[field_name]
+                if vocabulary_name and (
+                   vocabulary_name != getattr(field, 'vocabulary', None) and
+                   vocabulary_name != getattr(field, 'vocabularyName', None)):
+                    # Determine the widget to check for vocabulary there
+                    widgets = mergedTaggedValueDict(schema, WIDGETS_KEY)
+                    widget = widgets.get(field_name)
+                    if widget:
+                        widget = (isinstance(widget, basestring) and
+                                  resolveDottedName(widget) or widget)
+                        widget = widget and widget(field, self._request)
+                    else:
+                        widget = queryMultiAdapter((field, self._request),
+                                                   IFieldWidget)
+                    if getattr(widget, 'vocabulary', None) != vocabulary_name:
+                        return False
+                # Create mapping of all schema permissions
+                permissions = mergedTaggedValueDict(schema,
+                                                    WRITE_PERMISSIONS_KEY)
+                permission_name = permissions.get(field_name, None)
+                if permission_name is not None:
+                    permission = queryUtility(IPermission,
+                                              name=permission_name)
+                    if permission:
+                        return checker(permission.title, context)
+
+                # If the field is in the schema, but no permission is
+                # specified, fall back to the default edit permission
+                return checker(self.DEFAULT_PERMISSION, context)
+        else:
+            raise AttributeError('No such field: {0}'.format(field_name))
+
+
+class DXAddViewFieldPermissionChecker(DXFieldPermissionChecker):
+    """Permission checker for when we just have an add view"""
+
+    adapts(IAddForm)
+
+    def __init__(self, view):
+        self.context = view.context
+        # This may fail for views that aren't DefaultAddForm or
+        # DefaultAddView sub-classes, but they can register their own
+        # more specific adapters, if needed.
+        self.fti = getattr(view, 'fti', None)
+        if self.fti is None:
+            self.fti = view.ti
+        self._request = view.request
+
+    def _get_schemata(self):
+        fti = self.fti
+        yield fti.lookupSchema()
+        for schema in getAdditionalSchemata(portal_type=fti.getId()):
+            yield schema
