@@ -3,9 +3,22 @@
 from DateTime import DateTime
 from datetime import datetime
 from mock import Mock
+from plone.app.testing import TEST_USER_ID
+from plone.app.testing import TEST_USER_NAME
+from plone.app.testing import login
+from plone.app.testing import setRoles
+from plone.app.widgets.browser.vocabulary import VocabularyView
+from plone.app.widgets.testing import PLONEAPPWIDGETS_INTEGRATION_TESTING
 from plone.app.widgets.testing import TestRequest
 from plone.testing.zca import ZCML_DIRECTIVES
+from Products.Archetypes.atapi import BaseContent
+from Products.Archetypes.atapi import StringField
+from Products.Archetypes.atapi import Schema
+from Products.CMFCore.utils import getToolByName
 from zope.configuration import xmlconfig
+from zope.globalrequest import setRequest
+
+import json
 
 try:
     import unittest2 as unittest
@@ -73,8 +86,8 @@ class DateWidgetTests(unittest.TestCase):
                 'pattern_options': {
                     'date': {
                         'firstDay': 0,
-                        'min': [1913, 1, 1],
-                        'max': [2033, 1, 1],
+                        'min': [1914, 1, 1],
+                        'max': [2034, 1, 1],
                         'clear': u'Clear',
                         'format': 'mmmm d, yyyy',
                         'monthsFull': [u'January', u'February', u'March',
@@ -132,8 +145,8 @@ class DatetimeWidgetTests(unittest.TestCase):
                 'pattern_options': {
                     'date': {
                         'firstDay': 0,
-                        'min': [1913, 1, 1],
-                        'max': [2033, 1, 1],
+                        'min': [1914, 1, 1],
+                        'max': [2034, 1, 1],
                         'clear': u'Clear',
                         'format': 'mmmm d, yyyy',
                         'monthsFull': [u'January', u'February', u'March',
@@ -242,10 +255,44 @@ class SelectWidgetTests(unittest.TestCase):
             widget._base_args(self.context, self.field, self.request),
         )
 
+    def test_widget_orderable(self):
+        from plone.app.widgets.at import SelectWidget
+        widget = SelectWidget()
+        widget.multiple = True
+        widget.orderable = True
+        self.assertEqual(
+            {
+                'multiple': True,
+                'name': 'fieldname',
+                'pattern_options': {'orderable': True, 'separator': ';'},
+                'pattern': 'select2',
+                'value': (),
+                'items': [
+                    ('one', 'one'),
+                    ('two', 'two'),
+                    ('three', 'three')
+                ]
+            },
+            widget._base_args(self.context, self.field, self.request),
+        )
+
+    def test_process_form(self):
+        from plone.app.widgets.at import SelectWidget
+        widget = SelectWidget()
+        form = {'fieldname': 'aaa.bbb.ccc'}
+        self.assertEquals('aaa.bbb.ccc',
+                          widget.process_form(self.context,
+                                              self.field, form)[0])
+        widget.multiple = True
+        widget.separator = "."
+        self.assertEquals(['aaa', 'bbb', 'ccc'],
+                          widget.process_form(self.context,
+                                              self.field, form)[0])
 
 # TODO
 #class AjaxSelectWidgetTests(unittest.TestCase):
 
+
 class RelatedItemsWidgetTests(unittest.TestCase):
 
     layer = ZCML_DIRECTIVES
@@ -253,7 +300,7 @@ class RelatedItemsWidgetTests(unittest.TestCase):
     def setUp(self):
 
         self.request = TestRequest(environ={'HTTP_ACCEPT_LANGUAGE': 'en'})
-        self.context = Mock()
+        self.context = Mock(absolute_url=lambda: '')
         self.field = Mock()
 
         xmlconfig.file('configure.zcml', plone.uuid,
@@ -290,8 +337,10 @@ class RelatedItemsWidgetTests(unittest.TestCase):
                 'pattern_options': {
                     'folderTypes': ['SomeType'],
                     'separator': ';',
+                    'orderable': True,
                     'vocabularyUrl': '/@@getVocabulary?name='
-                                     'plone.app.vocabularies.Catalog',
+                                     'plone.app.vocabularies.Catalog'
+                                     '&field=fieldname',
                 },
             },
             widget._base_args(self.context, self.field, self.request),
@@ -334,24 +383,158 @@ class QueryStringWidgetTests(unittest.TestCase):
 
 class TinyMCEWidgetTests(unittest.TestCase):
 
+    layer = PLONEAPPWIDGETS_INTEGRATION_TESTING
+
     def setUp(self):
+        self.portal = self.layer['portal']
         self.request = TestRequest(environ={'HTTP_ACCEPT_LANGUAGE': 'en'})
-        self.context = Mock()
         self.field = Mock()
         self.field.getAccessor.return_value = lambda: 'fieldvalue'
         self.field.getName.return_value = 'fieldname'
 
     def test_widget(self):
+        # BBB: portal_tinymce is removed in Plone 5. Remove this check when
+        # Plone < 5 is no longer supported.
+        utility = getToolByName(self.portal, 'portal_tinymce', None)
+        if not utility:
+            return
         from plone.app.widgets.at import TinyMCEWidget
         widget = TinyMCEWidget()
-        self.context.portal_properties.site_properties\
-            .getProperty.return_value = 'utf-8'
-        self.assertEqual(
-            {
-                'name': 'fieldname',
-                'value': 'fieldvalue',
-                'pattern': 'tinymce',
-                'pattern_options': {},
-            },
-            widget._base_args(self.context, self.field, self.request),
-        )
+        self.field.widget = widget
+        base_args = widget._base_args(self.portal, self.field, self.request)
+        self.assertEqual(base_args['name'], 'fieldname')
+        self.assertEqual(base_args['value'], 'fieldvalue')
+        self.assertEqual(base_args['pattern'], 'tinymce')
+        # TODO: remove those lines
+        #self.assertEqual(base_args['pattern_options']['prependToUrl'],
+                         #'resolveuid/')
+        #self.assertEqual(base_args['pattern_options']['prependToUrl'],
+                         #'resolveuid/')
+        #self.assertEqual(base_args['pattern_options']['anchorSelector'],
+                         #self.portal.portal_tinymce.anchor_selector)
+
+
+class ArchetypesVocabularyPermissionTests(unittest.TestCase):
+
+    layer = PLONEAPPWIDGETS_INTEGRATION_TESTING
+
+    def setUp(self):
+        from plone.app.widgets.at import AjaxSelectWidget
+        self.request = TestRequest(environ={'HTTP_ACCEPT_LANGUAGE': 'en'})
+        setRequest(self.request)
+        self.portal = self.layer['portal']
+
+        login(self.portal, TEST_USER_NAME)
+        setRoles(self.portal, TEST_USER_ID, ['Manager'])
+
+        class TestAT(BaseContent):
+
+            schema = BaseContent.schema.copy() + Schema((
+                StringField(
+                    'allowed_field',
+                    vocabulary_factory='plone.app.vocabularies.PortalTypes',
+                    write_permission='View'),
+                StringField(
+                    'disallowed_field',
+                    vocabulary_factory='plone.app.vocabularies.PortalTypes',
+                    write_permission='View management screens'),
+                StringField(
+                    'default_field',
+                    vocabulary_factory='plone.app.vocabularies.PortalTypes'),
+                StringField(
+                    'allowed_widget_vocab',
+                    write_permission='View',
+                    widget=AjaxSelectWidget(
+                        vocabulary='plone.app.vocabularies.PortalTypes'),
+                )))
+
+        self.portal._setObject('test_at', TestAT('test_at'),
+                               suppress_events=True)
+
+        self.portal.test_at.manage_permission('View',
+                                              ('Anonymous',),
+                                              acquire=False)
+        self.portal.test_at.manage_permission('View management screens',
+                                              (),
+                                              acquire=False)
+        self.portal.test_at.manage_permission('Modify portal content',
+                                              ('Editor', 'Manager',
+                                               'Site Adiminstrator'),
+                                              acquire=False)
+
+    def test_vocabulary_field_allowed(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.PortalTypes',
+            'field': 'allowed_field',
+        })
+        data = json.loads(view())
+        self.assertEquals(len(data['results']),
+                          len(self.portal.portal_types.objectIds()))
+
+    def test_vocabulary_field_wrong_vocab_disallowed(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.Fake',
+            'field': 'allowed_field',
+        })
+        data = json.loads(view())
+        self.assertEquals(data['error'], 'Vocabulary lookup not allowed')
+
+    def test_vocabulary_field_disallowed(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.PortalTypes',
+            'field': 'disallowed_field',
+        })
+        data = json.loads(view())
+        self.assertEquals(data['error'], 'Vocabulary lookup not allowed')
+
+    def test_vocabulary_field_default_permission(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.PortalTypes',
+            'field': 'default_field',
+        })
+        # If the field is does not have a security declaration, the
+        # default edit permission is tested (Modify portal content)
+        setRoles(self.portal, TEST_USER_ID, ['Member'])
+        data = json.loads(view())
+        self.assertEquals(data['error'], 'Vocabulary lookup not allowed')
+
+        setRoles(self.portal, TEST_USER_ID, ['Editor'])
+        # Now access should be allowed, but the vocabulary does not exist
+        data = json.loads(view())
+        self.assertEquals(len(data['results']),
+                          len(self.portal.portal_types.objectIds()))
+
+    def test_vocabulary_field_default_permission_wrong_vocab(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.Fake',
+            'field': 'default_field',
+        })
+        setRoles(self.portal, TEST_USER_ID, ['Editor'])
+        # Now access should be allowed, but the vocabulary does not exist
+        data = json.loads(view())
+        self.assertEquals(data['error'], 'Vocabulary lookup not allowed')
+
+    def test_vocabulary_widget_vocab_allowed(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.PortalTypes',
+            'field': 'allowed_widget_vocab',
+        })
+        data = json.loads(view())
+        self.assertEquals(len(data['results']),
+                          len(self.portal.portal_types.objectIds()))
+
+    def test_vocabulary_missing_field(self):
+        view = VocabularyView(self.portal.test_at, self.request)
+        self.request.form.update({
+            'name': 'plone.app.vocabularies.PortalTypes',
+            'field': 'missing_field',
+        })
+        setRoles(self.portal, TEST_USER_ID, ['Member'])
+        with self.assertRaises(AttributeError):
+            view()
